@@ -1,7 +1,7 @@
 package Perinci::Result::Format::Lite;
 
-our $DATE = '2017-01-10'; # DATE
-our $VERSION = '0.23'; # VERSION
+our $DATE = '2017-05-24'; # DATE
+our $VERSION = '0.24'; # VERSION
 
 use 5.010001;
 #IFUNBUILT
@@ -56,6 +56,7 @@ sub __gen_table {
 
     $resmeta //= {};
 
+    # column names
     my @columns;
     if ($header_row) {
         @columns = @{$data->[0]};
@@ -148,26 +149,36 @@ sub __gen_table {
         my $tff   = $resmeta->{'table.fields'} or last;
         my $tffmt = $resmeta->{'table.field_formats'} or last;
 
-        # load required modules
-        for my $ffmt (@$tffmt) {
-            next unless $ffmt;
-            if ($ffmt eq 'iso8601') {
-                #require Time::Local;
+        my (@fmt_names, @fmt_opts); # index: column indexes
+        for my $i (0..$#columns) {
+            my $field_idx = $field_idxs[$i];
+            next unless $field_idx >= 0;
+            next unless defined $tffmt->[$field_idx];
+            if (ref($tffmt->[$field_idx]) eq 'ARRAY') {
+                $fmt_names[$i] = $tffmt->[$field_idx][0];
+                $fmt_opts [$i] = $tffmt->[$field_idx][1] // {};
+            } else {
+                $fmt_names[$i] = $tffmt->[$field_idx];
+                $fmt_opts [$i] = {};
             }
         }
 
         for my $i (0..$#{$data}) {
+            next if $i==0 && $header_row;
             my $row = $data->[$i];
             for my $j (0..$#columns) {
                 next unless defined $row->[$j];
                 my $field_idx = $field_idxs[$j];
+                #say "D:j=$j, field_idx=$field_idx";
                 next unless $field_idx >= 0;
-                my $ffmt = $tffmt->[$field_idx];
-                next unless $ffmt;
-                if ($ffmt eq 'iso8601_datetime' || $ffmt eq 'iso8601_date') {
+                my $fmt_name = $fmt_names[$j];
+                #say "D:j=$j fmt_name=$fmt_name";
+                next unless $fmt_name;
+                my $fmt_opts = $fmt_opts [$j];
+                if ($fmt_name eq 'iso8601_datetime' || $fmt_name eq 'iso8601_date') {
                     if ($row->[$j] =~ /\A[0-9]+\z/) {
                         my @t = gmtime($row->[$j]);
-                        if ($ffmt eq 'iso8601_datetime') {
+                        if ($fmt_name eq 'iso8601_datetime') {
                             $row->[$j] = sprintf(
                                 "%04d-%02d-%02dT%02d:%02d:%02dZ",
                                 $t[5]+1900, $t[4]+1, $t[3], $t[2], $t[1], $t[0]);
@@ -177,13 +188,16 @@ sub __gen_table {
                                 $t[5]+1900, $t[4]+1, $t[3]);
                         }
                     }
-                } elsif ($ffmt eq 'boolstr') {
+                } elsif ($fmt_name eq 'boolstr') {
                     $row->[$j] = $row->[$j] ? "yes" : "no";
-                } elsif ($ffmt eq 'sci2dec') {
+                } elsif ($fmt_name eq 'sci2dec') {
                     if ($row->[$j] =~ /\A(?:[+-]?)(?:\d+\.|\d*\.(\d+))[eE]([+-]?\d+)\z/) {
                         my $n = length($1 || "") - $2; $n = 0 if $n < 0;
                         $row->[$j] = sprintf("%.${n}f", $row->[$j]);
                     }
+                } elsif ($fmt_name eq 'percent') {
+                    my $fmt = $fmt_opts->{sprintf} // '%.2f%%';
+                    $row->[$j] = sprintf($fmt, $row->[$j] * 100);
                 }
             }
         }
@@ -395,7 +409,13 @@ sub format {
             } elsif (Data::Check::Structure::is_aos($data, {max=>$max})) {
                 return join("", map {"$_\n"} @$data);
             } elsif (Data::Check::Structure::is_aoaos($data, {max=>$max})) {
-                return __gen_table($data, 0, $res->[3], $format);
+                my $header_row = 0;
+                my $data = $data;
+                if ($res->[3]{'table.fields'}) {
+                    $data = [$res->[3]{'table.fields'}, @$data];
+                    $header_row = 1;
+                }
+                return __gen_table($data, $header_row, $res->[3], $format);
             } elsif (Data::Check::Structure::is_hos($data, {max=>$max})) {
                 $data = [map {[$_, $data->{$_}]} sort keys %$data];
                 unshift @$data, ["key", "value"];
@@ -464,7 +484,7 @@ Perinci::Result::Format::Lite - Format enveloped result
 
 =head1 VERSION
 
-This document describes version 0.23 of Perinci::Result::Format::Lite (from Perl distribution Perinci-Result-Format-Lite), released on 2017-01-10.
+This document describes version 0.24 of Perinci::Result::Format::Lite (from Perl distribution Perinci-Result-Format-Lite), released on 2017-05-24.
 
 =head1 SYNOPSIS
 
@@ -522,7 +542,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2017 by perlancar@cpan.org.
+This software is copyright (c) 2017, 2016, 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
